#include "optbrowser.h"

//#GENERATOR |void generateGlobals(ostream& cFILE)|cFILE
// ************************************************************************
// *                                                                      *
// *           Part of the Draft Proposal of Skeleton library             *
// *                                                                      *
// *                    (c) Nov 2001 J. Striegnitz                        *
// *                                                                      *
// ************************************************************************


template <typename T>
struct typeTraits {
  typedef T        id_t;
  typedef T        plain_t;
	typedef T*       pointer_t;
	typedef T&       reference_t;
	typedef const T  const_t;
	typedef const T* const_pointer_t;
	typedef const T& const_reference_t;
	typedef const T* const const_pointer_to_const_t;
	
	typedef const T& mkArg_t;
};

template <typename T>
struct typeTraits<T*> {
  typedef T*       id_t;
  typedef T        plain_t;
	typedef T*       pointer_t;
	typedef T&       reference_t;
	typedef const T  const_t;
	typedef const T* const_pointer_t;
	typedef const T& const_reference_t;
	typedef const T* const const_pointer_to_const_t;
	
	typedef T* mkArg_t;
};

template <typename T>
struct typeTraits<T&> {
  typedef T&       id_t;
  typedef T        plain_t;
	typedef T*       pointer_t;
	typedef T&       reference_t;
	typedef const T  const_t;
	typedef const T* const_pointer_t;
	typedef const T& const_reference_t;
	typedef const T* const const_pointer_to_const_t;
	
	typedef const T& mkArg_t;
};

template <typename T>
struct typeTraits<const T> {
  typedef const T  id_t;
  typedef T        plain_t;
	typedef T*       pointer_t;
	typedef T&       reference_t;
	typedef const T  const_t;
	typedef const T* const_pointer_t;
	typedef const T& const_reference_t;
	typedef const T* const const_pointer_to_const_t;
	
	typedef const T& mkArg_t;
};

template <typename T>
struct typeTraits<const T&> {
  typedef const T& id_t;
  typedef T        plain_t;
	typedef T*       pointer_t;
	typedef T&       reference_t;
	typedef const T  const_t;
	typedef const T* const_pointer_t;
	typedef const T& const_reference_t;
	typedef const T* const const_pointer_to_const_t;
	
	typedef const T& mkArg_t;
};

template <typename T>
struct typeTraits<const T*> {
  typedef const T*  id_t;
  typedef T         plain_t;
	typedef T*        pointer_t;
	typedef T&        reference_t;
	typedef const T   const_t;
	typedef const T*  const_pointer_t;
	typedef const T&  const_reference_t;
	typedef const T*  const const_pointer_to_const_t;
	
	typedef const T*  mkArg_t;
};

template <typename T>
struct curryArgMode {
  typedef typename typeTraits<T>::mkArg_t Type_t;	
// If you don't want to enforce constant references, you may use
// the following definition instead:
// typedef T Type_t;	
};

//#END_GENERATOR

// ============================================================================
//
//                     C U R R Y   F U NC T I O N S 
// 
// ============================================================================
//#GENERATOR |void generateCurry(int MAX_ORDER, ostream& cFILE)|cFILE
# for (int i=0 ; i< MAX_ORDER ; i++) {
  template <`["typename A%d",0,i,",",true] typename R,typename F>
  class Fct`(i) {
	  F f;
	public:	
	  // Constructor, copy constructor and destructor
	  Fct`(i)(const F& _f)        : f(_f)    {}
		Fct`(i)(const Fct`(i)& rhs)    : f(rhs.f) {}
		~Fct`(i)()                             {}
		
# if (i==0) {
    inline operator R() const {
		  return f();
		}		
# } else { 
		
    // Closures
		struct closureT {
  	  F op;
`["      A%d pa%d;",0,i,"\n"]		
      closureT(const F& _op,
			                `["typename curryArgMode<A%d>::Type_t _pa%d",0,i]
										 ) : op(_op),`["pa%d(_pa%d)",0,i] {}
      closureT(const closureT& rhs) : op(rhs.op),
			                `["pa%d(rhs.pa%d)",0,i] {}
      inline R operator()() const {
			  return op(`["pa%d",0,i]);
			}
		};

    typedef Fct0<R, closureT> closure_t;
    inline closure_t closure(`["typename curryArgMode<A%d>::Type_t a%d",0,i]) const {
		  return closure_t( closureT(f,`["a%d",0,i]) );
		}
		
# }
		// Full application -> delegate to instance of F
		inline 
		R operator()(`["typename curryArgMode<A%d>::Type_t a%d",0,i]) const {
		  return f(`["a%d",0,i]);
		}
		

	
	  // Support for partial application 
# for (int j=1 ; j < i ; j ++) {
    struct PartialAppl`(j) { // F applied to `(j) arguments
		  F f;
`["      A%d pa%d;",0,j,"\n"]
      PartialAppl`(j) (const F& _f,
			                `["typename curryArgMode<A%d>::Type_t _pa%d",0,j]
										 ) : f(_f),`["pa%d(_pa%d)",0,j] {}
      PartialAppl`(j) (const PartialAppl`(j)& rhs) : f(rhs.f),
			                `["pa%d(rhs.pa%d)",0,j] {}
      inline R operator()(`["typename curryArgMode<A%d>::Type_t a%d",j,i]) const {
			  return f(`["pa%d",0,j],`["a%d",j,i]);
			}			
		}; 
		
		typedef Fct`(i-j)<`["A%d",j,i], R,PartialAppl`(j)> ffunc`(j)_t;
 	  inline 
	  ffunc`(j)_t operator()(`["typename curryArgMode<A%d>::Type_t a%d",0,j]) const {
	    return ffunc`(j)_t(PartialAppl`(j)(f, `["a%d",0,j]) );
	  }
# }
	};	
	// curry functions
  template <`["typename A%d",0,i,",",true] typename R>
	inline
	Fct`(i)<`["A%d",0,i,",",true] R,R (*)(`["A%d",0,i])> curry(R (* cppfunc)(`["A%d",0,i]) ) {
	  return Fct`(i)<`["A%d",0,i,",",true] R,R (*)(`["A%d",0,i])>( cppfunc );
	}

	// stream inserters
  template <`["typename A%d",0,i,",",true] typename R,typename F>
	inline
	ostream& operator<<(ostream& os,const Fct`(i)<`["A%d",0,i,",",true] R,F> &) {
	  os << "`(i)-ary function";
		return os;
	}
# }
//#END_GENERATOR

//#GENERATOR | void generateDynamicFct(int MAX_ORDER,ostream& cFILE)|cFILE
# for(int i=0 ; i<MAX_ORDER ; i++) {
  template <`["typename A%d",0,i,",",true] typename R>
  struct anyOp`(i) {  
	  anyOp`(i)() {}
		anyOp`(i)(const anyOp`(i)& rhs) {}
	  inline 
		virtual R operator()(`["typename curryArgMode<A%d>::Type_t a%d",0,i]) const = 0;
	};
	
	template <`["typename A%d",0,i,",",true] typename R,typename F>
	struct anyFunc`(i) : public anyOp`(i)<`["A%d",0,i,",",true] R> {
	  typedef Fct`(i)<`["A%d",0,i,",",true] R, F> ffunc_t;
		ffunc_t f;
		anyFunc`(i)(const ffunc_t& _f) : f(_f) {}
		anyFunc`(i)(const anyFunc`(i)& rhs) : f(rhs.f) {}
		inline 
		virtual R operator()(`["typename curryArgMode<A%d>::Type_t a%d",0,i]) const {
		  return f(`["a%d",0,i]);
		}
	};
	
	template <`["typename A%d",0,i,",",true] typename R>
	struct DFct`(i) {
	  int* refCount;
		anyOp`(i)<`["A%d",0,i,",",true] R>* op;
		
		template <typename F>
		DFct`(i)(const Fct`(i)<`["A%d",0,i,",",true] R,F>& f) {
		  op       = new anyFunc`(i)<`["A%d",0,i,",",true] R,F>(f);
			refCount = new int(1);
		}
	  DFct`(i)(const DFct`(i)& rhs) : refCount(rhs.refCount),op(rhs.op) { (*refCount)++ ; }
	  ~DFct`(i)() {
		  if ( --(*refCount) == 0 ) {
			  delete refCount;
			  delete op;
			}
		}
		DFct`(i)<`["A%d",0,i,",",true] R> operator=(const DFct`(i)<`["A%d",0,i,",",true] R>& rhs) {
		  if (rhs.op == op) return *this;
			if ( --(*refCount) == 0 ) {
			  delete refCount;
				delete op;
			}
			op       = rhs.op;
			refCount = rhs.refCount;
			(*refCount)++;
			return *this;
		}
		template <typename F>
		DFct`(i)<`["A%d",0,i,",",true] R> operator=(const Fct`(i)<`["A%d",0,i,",",true] R,F>& rhs) {
		  if (rhs.op == op) return *this;
			if ( --(*refCount) == 0 ) {
			  delete refCount;
				delete op;
			}
			op       = new anyFunc`(i)<`["A%d",0,i,",",true] R,F>(f);
			refCount = new int(1);
			(*refCount)++;
			return *this;
		}
		
		inline 
		R operator()(`["typename curryArgMode<A%d>::Type_t a%d",0,i]) const {
		  return (*op)(`["a%d",0,i]);
		}

# for (int j=1 ; j < i ; j ++) {
    struct PartialAppl`(j) { // F applied to `(j) arguments
		  anyOp`(i)<`["A%d",0,i,",",true] R> op;
`["      A%d pa%d;",0,j,"\n"]
      PartialAppl`(j) (const anyOp`(i)<`["A%d",0,i,",",true] R>& op ,
			                `["typename curryArgMode<A%d>::Type_t _pa%d",0,j]
										 ) : op(_op),`["pa%d(_pa%d)",0,j] {}
      PartialAppl`(j) (const PartialAppl`(j)& rhs) : op(rhs.op),
			                `["pa%d(rhs.pa%d)",0,j] {}
      inline R operator()(`["typename curryArgMode<A%d>::Type_t a%d",j,i]) const {
			  return (*op)(`["pa%d",0,j],`["a%d",j,i]);
			}			
		}; 
		typedef Fct`(i-j)<`["A%d",j,i], R,PartialAppl`(j)> ffunc`(j)_t;
 	  inline 
	  ffunc`(j)_t operator()(`["typename curryArgMode<A%d>::Type_t a%d",0,j]) const {
	    return ffunc`(j)_t(PartialAppl`(j)(*op, `["a%d",0,j]) );
	  }
# }		
		
	};
# }
//#END_GENERATOR

int main(int argc,char** argv) {
  COptBrowser opt(argc,argv);
	
	int MaxArity;
	
	if (! opt.getOpt("-n","%d",&MaxArity) )
	  MaxArity = 7;
	
  generateGlobals(cout);
	generateCurry(MaxArity,cout);
	generateDynamicFct(MaxArity,cout);
}
