#include "Skeleton.h"
#include "math.h"

inline int negate(const int a){return -a;}

inline int add(const int a, const int b){return a+b;}
 
template <class C>
inline C skprod(const DistributedMatrix<C>& A, const DistributedMatrix<C>& B, int i, int j, C Cij){
  C sum = Cij;
  for (int k=0; k<A.getLocalRows(); ++k)
     //sum += A.getGlobalLocal(i,k) * B.getLocalGlobal(k,j);
     sum +=  B.getLocalGlobal(k,j) * A.getGlobalLocal(i,k);
  return sum;}

template <class C> // using algorithm of Gentleman based on torus topology
DistributedMatrix<C> matmult(DistributedMatrix<C> A,DistributedMatrix<C> B){
    // assumption:  A, B have same square shape
    A.rotateRows(& negate);
    //A.show();
    B.rotateCols(& negate); 
    //B.show();
    DistributedMatrix<C> R(A.getRows(),A.getCols(),0, A.getBlocksInCol(), A.getBlocksInRow());
    for (int i=0; i< A.getBlocksInRow(); ++i){  
      typedef C (*skprod_t)(const DistributedMatrix<C>&, 
                            const DistributedMatrix<C>&, int, int, C);
      R.mapIndexInPlace(curry((skprod_t)skprod<C>)(A)(B));
      A.rotateRows(-1);
      B.rotateCols(-1);}
    return R;}

int main(int argc, char **argv){
//*************************************************************
// Test: matrix multiplication
// *************************************************************
  try{
    InitSkeletons(argc,argv);

    int sqrtp = (int) (sqrt(sk_numprocs)+0.1);
    DistributedMatrix<int> A(Problemsize,Problemsize,& add,sqrtp,sqrtp);
    DistributedMatrix<int> B(Problemsize,Problemsize,& add,sqrtp,sqrtp);   

#if Output == 1
    if (sk_myid == 0) {cout << "A:" << endl << flush;} A.show();
    if (sk_myid == 0) {cout << "B:" << endl << flush;} B.show();
#endif

    DistributedMatrix<int> C = matmult(A,B);

#if Output == 1
    if (sk_myid == 0) {cout << "A x B:" << endl << flush;}
    C.show();
#endif

    TerminateSkeletons();}
  catch(Exception&){cout << "Exception" << endl << flush;};                  
}
