#include "Skeleton.h"
#include "math.h"

// general assumption: higher bits to the left of lower bits
int log2p;
int log2size;
const double pi = 3.141592653589793238462643383;    

struct complex{double real; double imag;
  complex(double r, double i): real(r), imag(i){}
  complex(){}};

inline complex operator+(complex c1, complex c2){return complex(c1.real+c2.real,c1.imag+c2.imag);}
inline complex operator*(complex c1, complex c2){
  return complex(c1.real*c2.real-c1.imag*c2.imag, c1.real*c2.imag+c1.imag*c2.real);}
ostream& operator<<(ostream& os, const complex& c){
  os << c.real << " + i*" << c.imag;
  return os;}

inline complex copy(const DistributedArray<complex>& A, int i, complex Ai){return A.get(i);}

inline complex init(const DistributedArray<double>& A, int i){return complex(A.get(i),0.0);}

inline int bitcomplement(int k, int i){return i ^ (1<<k);} 

inline complex omega(int j, int i){
  int b = i >> (log2size-j-1);   int b2 = 0;
  for (int l=0; l<=j; l++){
    b2 = (b & 1) ? 2*b2+1  :  2*b2;
    b >>= 1;}
  double v = 2.0 * pi/ Problemsize * (b2 << (log2size-j-1)); 
  return complex(cos(v), sin(v));}

inline complex combine(const DistributedArray<complex>& T, int j, int i, complex Ai){
  return (i & (1<<log2size-1-j)) ? T.get(i) + omega(j,i) * Ai
                                 : Ai + omega(j,i) * T.get(i);} 

inline complex fetch(const DistributedArray<complex>& R, int j, int i, complex Ti){
  return R.get(bitcomplement(log2size-1-j,i));}

DistributedArray<complex> fft(const DistributedArray<double>& A){
    int j;
    DistributedArray<complex> R(Problemsize,curry(init)(A));
    DistributedArray<complex> T(Problemsize);  
    for (j=0; j<log2p; j++){
      T.mapIndexInPlace(curry(copy)(R));
      T.permutePartition(curry(bitcomplement)(log2p-1-j));
      R.mapIndexInPlace(curry(combine)(T)(j));}
    for (j=log2p; j<log2size; j++){
      T.mapIndexInPlace(curry(fetch)(R)(j));
      R.mapIndexInPlace(curry(combine)(T)(j));}
    return R;}

int main(int argc, char **argv){
//*********************************************************************************
// Test: FFT (algorithm (roughly) according to:
// M. Quinn: Parallel Computing - Theory and Practice, McGraw-Hill, 1994, p. 207)
//*********************************************************************************
  try{
    InitSkeletons(argc,argv);
    log2p = log2(sk_numprocs);
    log2size  = log2(Problemsize); 
 
    DistributedArray<double> A(Problemsize,1.0);
    DistributedArray<complex> B = fft(A); 

#if Output == 1
    B.show(); 
#endif

    TerminateSkeletons();}
  catch(Exception&){cout << "Exception" << endl << flush;}                  
}
