#include "Skeleton.h"

inline double init(const int a, const int b){return (a==b) ? 1.0 : 2.0;}

inline double copyPivot(const DistributedMatrix<double>& A, int k, int i, int j, double Pij){
  return A.isLocal(k,k) ? A.get(k,j)/A.get(k,k) : 0;}

inline double pivotOp(const DistributedMatrix<double>& A,  
              const DistributedMatrix<double>& Pivot, 
            int k, int i, int j, double Pij){
  double result = (i == k) ? Pivot.get(i/A.getLocalRows(),j)
                          : A.get(i,j) - A.get(i,k) * Pivot.get(i/A.getLocalRows(),j);
  return result;}

// idea: - each processor gets a sequence of rows of matrix A
//       - repeatedly the pivot row is broadcasted to everybody
//         and each processor executes the pivot operation on its partition of A
DistributedMatrix<double> gauss(const DistributedMatrix<double>& A){
  // for simplicity assuming pivot != 0
  DistributedMatrix<double> B(Problemsize,Problemsize+1,0.0,sk_numprocs,1);
  DistributedMatrix<double> ACopy(A); 
  DistributedMatrix<double> Pivot(sk_numprocs,Problemsize+1,0.0,sk_numprocs,1);
  DistributedMatrix<double>* Bptr = &B;
  DistributedMatrix<double>* Aptr = &ACopy;
  DistributedMatrix<double>* AuxPtr;
  for (int k=0; k<Problemsize; k++){
    Pivot.mapIndexInPlace(curry(copyPivot)(*Aptr)(k));
    Pivot.broadcastPartition(k/(*Aptr).getLocalRows(),0); 
    (*Bptr).mapIndexInPlace(curry(pivotOp)(*Aptr)(Pivot)(k)); 
    AuxPtr = Bptr; Bptr = Aptr; Aptr = AuxPtr;}
  return *Aptr;} 

int main(int argc, char **argv){
//*************************************************************
// Test: Gaussian Elimination
// *************************************************************
  try{
    InitSkeletons(argc,argv);
   
    DistributedMatrix<double> A(Problemsize,Problemsize+1,& init,sk_numprocs,1);

#if Output == 1
    if (sk_myid == 0) {cout << "A:" << endl << flush;} 
    A.show();
#endif

    DistributedMatrix<double> B = gauss(A);

#if Output == 1
    if (sk_myid == 0) {cout << "Result of Gaussian Elimination:" << endl << flush;}
    B.show();
#endif

    TerminateSkeletons();}
  catch(Exception&){cout << "Exception" << endl << flush;}               
}
